#ifndef GLOBAL_H
#define GLOBAL_H

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <dirent.h>

#include <mil1553ud_library.h>

#define VERSION_APP                     "28.05.2021"

/// \brief каталог с конфигами тестов
#define STR_CONFIG                      "./config/"
/// \brief каталог с логами
#define STR_LOGS                        "./logs/"
/// \brief каталог с mem файлами
#define STR_MEM                         "./mem/"
#define STR_DEV                         "/dev/"
/// \brief длина имени каталога config
#define LEN_STR_CONFIG                  9
/// \brief длина имени config файла
#define MAX_LEN_CONFIG_FILE             90
/// \brief длина полного имени config файла
#define MAX_LEN_FULL_CONFIG_FILE        MAX_LEN_CONFIG_FILE + LEN_STR_CONFIG
/// \brief длина имени log файла
#define LOGGER_FILENAME_LENGTH          160
/// \brief длина имени mem файла
#define MEM_FILENAME_LENGTH             160

///
/// \brief loadMemTxtFile чтение текстового mem файла с диска
/// \param nameFile имя файла
/// \param arrayWords массив 32х разрядных слов
/// \param count кразмер массива
/// \return результат операции
///
int loadMemTxtFile(const char* nameFile, unsigned int* arrayWords, int count);

///
/// \brief readline чтение строки
/// \param stream указатель текущего положения в файле
/// \param line указатель на строку
///
void readline(FILE* stream, char** line);
///
/// \brief constructLogFilename сгенерить имя лог файла
/// \param filename полное имя лог файла
/// \param length длина
/// \param testname имя теста
///
void constructLogFilename(char* filename, int length, char* testname);

/// \remark LOGGER

///
/// \brief LOGGER_FILE дескриптор файла логов
///
extern FILE* LOGGER_FILE;
/// \brief открыть лог файл
#define LOGGER_OPEN(filename) \
    LOGGER_FILE = fopen(filename, "w");
/// \brief закрыть лог файл
#define LOGGER_CLOSE() \
    fclose(LOGGER_FILE)
/// \brief сбросить лог файл буфер на диск
#define LOGGER_FLUSH() \
    fflush(LOGGER_FILE)
/// \brief принт в лог и в терминал
#define LOGGER_PRINT(fmt, args...) \
    printf(fmt, args); \
    fprintf(LOGGER_FILE, fmt, args)
/// \brief принт строки в лог и в терминал
#define LOGGER_PRINT_STR(str) \
    printf("%s", str); \
    fprintf(LOGGER_FILE, "%s", str)

//#define LOGGER_PRINT(fmt, args...)
//    printf(fmt, args)

# define timersub(a, b, result)						      \
  do {									      \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;			      \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;			      \
    if ((result)->tv_usec < 0) {					      \
      --(result)->tv_sec;						      \
      (result)->tv_usec += 1000000;					      \
    }									      \
  } while (0)

#endif // GLOBAL_H
